/*****************************************************************
 * Copyright (c) ParaSoft Corporation 1991.  All rights reserved.*
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF ParaSoft       *
 * The copyright notice above does not evidence any              *
 * actual or intended publication of such source code.           *
 *****************************************************************/

#include "extern.h"

Node *Node_0, *Node_1;
Node *Node_SubscriptList_1;
Node *Node_Then;

make_init ()  /* initializes node constants */
{
	Node_0 = make (INTEGER_CONSTANT);
	Node_0->com.str = "0";
	Node_0->com.typ = typ_malloc ();
	Node_0->com.typ->etype = Etype_INTEGER_0;

	Node_1 = make (INTEGER_CONSTANT);
	Node_1->com.str = "1";
	Node_1->com.typ = typ_malloc ();
	Node_1->com.typ->etype = Etype_INTEGER_0;

	Node_SubscriptList_1 = make (SUBSCRIPT_LIST);
	Node_SubscriptList_1->SubscriptList.Subscript = make (SUBSCRIPT);
	Node_SubscriptList_1->SubscriptList.Subscript->Subscript.Expr = Node_1;

	Node_Then = make_Name ("THEN");
}
/************************************************************************/
Node *make_IntegerConstant (i)
int i;
{
	Node *node;

	node = make (INTEGER_CONSTANT);
	sprintf (serr, "%d", i);
	node->com.str = U_str_copy (serr);
	node->com.typ = typ_malloc ();
	node->com.typ->etype = Etype_INTEGER_0;

	return node;
}

Node *make_Statement (label, stmt)
Node *label, *stmt;
{
	Node *statement;

	statement = make (STATEMENT);
	statement->Statement.IntegerConstant = label;
	statement->Statement.Stmt = stmt;
	statement->Statement.Eos = &Eos;
	
	return statement;
}

Node *make_Name (str)
char *str;
{
	Node *node;

	node = make (NAME);
	node->com.str = str;

	return node;
}

Node *make_KeyName (str)
char *str;
{
	Node *node;

	node = make (KEY_NAME);
	node->com.str = str;

	return node;
}

Node *make_AssignementStmt (var, expr)
Node *var, *expr;
{
	Node *node;

	node = make (ASSIGNEMENT_STMT);
	node->AssignementStmt.Variable = var;
	node->AssignementStmt.Eq = &Eq;
	node->AssignementStmt.Expr = expr;

	return node;
}

Node *make_DumExpr_parse (a1, op, a2)
Node *a1, *op, *a2;
{
	Node *expr;

	expr = make_DumExpr (a1, op, a2);
	expr = parse_DumExpr (expr);

	return expr;
}

Node *make_DumExpr (a1, op, a2)  /* calculates also Typ */
Node *a1, *op, *a2;
{
	Node *expr;

	if (op == NN && a2 == NN)
		return a1;
	if (op == NN && a1 == NN)
		return a2;

	expr = make (DUM_EXPR);
	expr->DumExpr.Expr = a1;
	expr->DumExpr.Op = op;
	expr->DumExpr.Expr1 = a2;

	return expr;
}

Node *make_DumExpr_1_PIN (a1)  /* Puts () around the expr if necessary */
Node *a1;
{
	Node *expr;

	if (a1->com.type == DUM_EXPR && a1->DumExpr.Lp == NN) {
		expr = make_DumExpr_paren (a1, NN, NN);
		expr->com.typ = a1->com.typ;
	}
	else
		expr = a1;

	return expr;
}

Node *make_DumExpr_write_paren (a1, op, a2)
Node *a1, *op, *a2;
{
	Node *expr;

	expr = make (DUM_EXPR);
	expr->DumExpr.Lp = &Lp;
	expr->DumExpr.Expr = a1;
	expr->DumExpr.Op = op;
	expr->DumExpr.Expr1 = a2;
	expr->DumExpr.Rp = &Rp;

	return expr;
}

Node *make_DumExpr_write (a1, op, a2)
Node *a1, *op, *a2;
{
	Node *expr;

	expr = make (DUM_EXPR);
	expr->DumExpr.Expr = a1;
	expr->DumExpr.Op = op;
	expr->DumExpr.Expr1 = a2;

	return expr;
}

Node *make_DumExpr_paren (a1, op, a2)  /* () around the expression */
Node *a1, *op, *a2;
{
	Node *expr;

	expr = make (DUM_EXPR);
	expr->DumExpr.Lp = &Lp;
	expr->DumExpr.Expr = a1;
	expr->DumExpr.Op = op;
	expr->DumExpr.Expr1 = a2;
	expr->DumExpr.Rp = &Rp;
	if (a2 == NN)
		expr->com.typ = a1->com.typ;
	else
		expr->com.typ = typ_expr (a1, op, a2);

	return expr;
}

Node *make_DumExpr_PIN (a1, op, a2) /* () around the operands if necessary */
Node *a1, *op, *a2;
{
	Node *expr;

	if (op == NN && a2 == NN)
		return a1;
	if (op == NN && a1 == NN)
		return a2;

	expr = make (DUM_EXPR);
	if (a1 != NN && a1->com.type == DUM_EXPR && a1->DumExpr.Lp == NN
	&& op != &Plus && op != &Minus)
		expr->DumExpr.Expr = make_DumExpr_paren (a1, NN, NN);
	else
		expr->DumExpr.Expr = a1;

	if (a1 != NN || op != &Plus)
		expr->DumExpr.Op = op;

	if (a2 != NN && a2->com.type == DUM_EXPR && a2->DumExpr.Lp == NN)
		expr->DumExpr.Expr1 = make_DumExpr_paren (a2, NN, NN);
	else
		expr->DumExpr.Expr1 = a2;

	expr->com.typ = typ_expr (a1, op, a2);

	return expr;
}

Node *make_DumExpr_PIN_no_typ (a1, op, a2) /* () around the operands if necessary */
Node *a1, *op, *a2;
{
	Node *expr;

	if (op == NN && a2 == NN)
		return a1;
	if (op == NN && a1 == NN)
		return a2;

	expr = make (DUM_EXPR);
	if (a1 != NN && a1->com.type == DUM_EXPR && a1->DumExpr.Lp == NN
	&& op != &Plus && op != &Minus)
		expr->DumExpr.Expr = make_DumExpr_paren (a1, NN, NN);
	else
		expr->DumExpr.Expr = a1;

	if (a1 != NN || op != &Plus)
		expr->DumExpr.Op = op;

	if (a2 != NN && a2->com.type == DUM_EXPR && a2->DumExpr.Lp == NN)
		expr->DumExpr.Expr1 = make_DumExpr_paren (a2, NN, NN);
	else
		expr->DumExpr.Expr1 = a2;

	return expr;
}

Node *make_Subscript (expr, expr1, expr2)
Node *expr, *expr1, *expr2;
{
	Node *sub;

	sub = make (SUBSCRIPT);
	sub->Subscript.Expr = expr;
	if (expr1 != NN) {
		sub->Subscript.Colon = &Colon;
		sub->Subscript.Expr1 = expr1;
	}
	if (expr2 != NN) {
		sub->Subscript.Colon1 = &Colon;
		sub->Subscript.Expr2 = expr2;
	}

	return sub;
}

Node *make_opt7 (sublist)
Node *sublist;
{
	Node *opt7;

	opt7 = make (OPT_7);
	if (sublist != NN) {
		opt7->opt7.Lp = &Lp;
		opt7->opt7.SubscriptList = sublist;
		opt7->opt7.Rp = &Rp;
	}

	return opt7;
}

Node *make_EntityDecl (name, opt7)
Node *name, *opt7;
{
	Node *entity;

	entity = make (ENTITY_DECL);
	entity->EntityDecl.Name = name;
	entity->EntityDecl.opt7 = opt7;

	return entity;
}

Node *make_ArraySection (name, sublist)
Node *name, *sublist;
{
	Node *array;

	array = make (ARRAY_SECTION);
	array->ArraySection.Name = name;
	if (sublist != NN) {
		array->ArraySection.Lp = &Lp;
		array->ArraySection.SubscriptList = sublist;
		array->ArraySection.Rp = &Rp;
	}
	array->com.str = name->com.str;

	return array;
}

Node *make_DumStmt (stmt, code)
Node *stmt, *code;
{
	Node *dumstmt;

	dumstmt = make (DUM_STMT);
	dumstmt->DumStmt.Stmt = stmt;
	dumstmt->DumStmt.Code = code;

	return dumstmt;
}

Node *make_Code (str)
char *str;
{
	Node *code;

	code = make (CODE);
	code->com.str = str;

	return code;
}

Node *make_List1 (value, num, type)
Node *value[];
int num, type;
{
	Node *listhead, *list = NN, *oldlist;
	int i;

	for (i = 0; i < num; i++) {
		oldlist = list;
		list = make (type);
		list->lst.value = value[i];
		if (i != num - 1)
			list->lst.Comma = &Comma;
		if (oldlist != NN)
			oldlist->lst.next = list;

		if (i == 0)
			listhead = list;
	}
	
	return listhead;
}

Node *make_List (value, num, type)
Node *value[];
int num, type;
{
	Node *listhead, *list = NN, *oldlist;
	int i;

	for (i = 0; i < num; i++) {
		oldlist = list;
		list = make (type);
		list->lst.value = value[i];
		if (i)
			list->lst.Comma = &Comma;
		if (oldlist != NN)
			oldlist->lst.next = list;

		if (i == 0)
			listhead = list;
	}
	
	return listhead;
}

Node *make_CallStmt (name, sublist)
Node *name, *sublist;
{
	Node *node;

	node = make (CALL_STMT);
	node->CallStmt.Call = &Call;
	node->CallStmt.Name = name;
	node->CallStmt.Lp = &Lp;
	node->CallStmt.SubscriptList = sublist;
	node->CallStmt.Rp = &Rp;

	node->com.str = name->com.str;

	return node;
}

Node *make_SubscriptList (expr)
Node *expr;
{
	Node *sub, *list;

	sub = make_Subscript (expr, NN, NN);
	list = make_List (&sub, 1, SUBSCRIPT_LIST);

	return list;
}
	
Node *make_FunctionReference (name, sublist)
Node *name, *sublist;
{
	Node *node;

	node = make (FUNCTION_REFERENCE);
	node->FunctionReference.Name = name;
	node->FunctionReference.Lp = &Lp;
	node->FunctionReference.SubscriptList = sublist;
	node->FunctionReference.Rp = &Rp;

	node->com.str = name->com.str;

	return node;
}

Node *make_DumList (num, list)
int num;
Node **list;
{
	Node *node;

	node = make (DUM_LIST);
	node->DumList.num = num;
	node->DumList.list= list;

	return node;
}

Node *make_LogicalConstant (str)
char *str;
{
	Node *node;

	node = make (LOGICAL_CONSTANT);
	node->com.str = str;

	return node;
}

Node *make_EntityDecl1 (decl, charlen, initexpr)
Node *decl, *charlen, *initexpr;
{
	Node *node;

	node = make (ENTITY_DECL_1);
	node->EntityDecl1.EntityDecl = decl;
	node->EntityDecl1.OptCharLength = charlen;
	node->EntityDecl1.OptInitializationExpr = initexpr;

	return node;
}

Node *make_EndStmt ()
{
	Node *node;

	node = make (END_STMT);
	node->EndStmt.End = &End;

	return node;
}

Node *make_EndDoStmt ()
{
	Node *node;

	node = make (END_DO_STMT);
	node->EndDoStmt.EndDo = &EndDo;

	return node;
}

Node *make_Variable (Name)
Node *Name;
{
	Node *node;

	node = make (VARIABLE);
	node->Variable.Name = Name;

	return node;
}

Node *make_DoStmt (LoopControl)
Node *LoopControl;
{
	Node *node;

	node = make (DO_STMT);
	node->DoStmt.Do = &Do;
	node->DoStmt.LoopControl = LoopControl;

	return node;
}

Node *make_ContinueStmt ()
{
	Node *node;

	node = make (CONTINUE_STMT);
	node->ContinueStmt.Continue = &Continue;

	return node;
}

Node *make_ContinueStatement (label)
Node *label;
{
	Node *node;

	return make_Statement (label, make_ContinueStmt ());
}

Node *make_GotoStmt (label)
Node *label;
{
	Node *node;

	node = make (GOTO_STMT);
	node->GotoStmt.GoTo = &GoTo;
	node->GotoStmt.Label = label;

	return node;
}

Node *make_IfThenStmt (expr)
Node *expr;
{
	Node *node;

	node = make (IF_THEN_STMT);
	node->IfThenStmt.If = &If;
	node->IfThenStmt.Lp = &Lp;
	node->IfThenStmt.Expr = expr;
	node->IfThenStmt.Rp = &Rp;
	node->IfThenStmt.Name = Node_Then;

	return node;
}

Node *make_ElseIfStmt (expr)
Node *expr;
{
	Node *node;

	node = make (ELSE_IF_STMT);
	node->ElseIfStmt.ElseIf = &ElseIf;
	node->ElseIfStmt.Lp = &Lp;
	node->ElseIfStmt.Expr = expr;
	node->ElseIfStmt.Rp = &Rp;
	node->ElseIfStmt.Name = Node_Then;

	return node;
}

Node *make_EndIfStmt ()
{
	Node *node;

	node = make (END_IF_STMT);
	node->EndIfStmt.EndIf = &EndIf;

	return node;
}

Node *make_ElseStmt ()
{
	Node *node;

	node = make (ELSE_STMT);
	node->ElseStmt.Else = &Else;

	return node;
}

Node *make_ImpliedDoControl (name, e1, e2, e3)
Node *name, *e1, *e2, *e3;
{
	Node *node;

	node = make (IMPLIED_DO_CONTROL);
	node->ImpliedDoControl.Name = name;
	node->ImpliedDoControl.Eq = &Eq;
	node->ImpliedDoControl.Expr = e1;
	node->ImpliedDoControl.Comma = &Comma;
	node->ImpliedDoControl.Expr1 = e2;

	if (e3 != NN) {
		node->ImpliedDoControl.Comma1 = &Comma;
		node->ImpliedDoControl.Expr2 = e3;
	}

	return node;
}

Node *make_ImpliedDo (ivl, idc)
Node *ivl, *idc;
{
	Node *node;

	node = make (IMPLIED_DO);
	node->ImpliedDo.Lpeqi = &Lpeqi;
	node->ImpliedDo.ImpliedValueList = ivl;
	node->ImpliedDo.Comma = &Comma;
	node->ImpliedDo.ImpliedDoControl = idc;
	node->ImpliedDo.Rp = &Rp;

	return node;
}

/*
Node *make_ ()
Node *;
{
	Node *node;

	node = make ();
	node->. = ;

	return node;
}
*/
